/**
 * @(#)examiner.cpp 2000/12/13
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#include "tsidexaminer.h"

#include <iostream.h>

// paramenter field
const int P_NOTHING   = 0x000;  // nothing passed
const int P_UNSORTED  = 0x001;  // unsorted list
const int P_SORTED    = 0x002;  // sorted list
const int P_AUTHOR    = 0x004;  // author list
const int P_TUNE      = 0x008;  // single tune list
const int P_SONG      = 0x010;  // songs (files) list
const int P_TIME      = 0x020;  // listen time form
const int P_NUMBER    = 0x040;  // number of listen form
const int P_PONDERATE = 0x080;  // ponderate form (listen/number)
const int P_HELP      = 0x100;  // help of commands
const int P_ERROR     = 0x200;  // error in parsing parameters

TSIDExaminer examiner;

/**
 * Get parameters summary as an integer value
 *
 * @param argc the numbr of passed string
 * @param argv the strings of parameters
 * @return an integer that summary all the options available
 */
int getParameter(int argc, char** argv) {
  int value=P_NOTHING;                       // returned value;

  // parse command line arguments
  int a=1;
  while ((a<argc) && (argv[a] != NULL))	{
    if (argv[a][0]=='-') {
      int b=1;
      while (argv[a][b]!='\0') {
        switch (argv[a][b]) {
          case 'u':                          // unsorted list
            value|=P_UNSORTED;
            break;
          case 's':                          // sorted list
            value|=P_SORTED;
            break;
          case 'a':                          // author list
            value|=P_AUTHOR;
            break;
          case 't':                          // single tune list
            value|=P_TUNE;
            break;
          case 'f':                          // songs (file) list
            value|=P_SONG;
            break;
          case 'l':                          // listen time form
            value|=P_TIME;
            break;
          case 'n':                          // number of listen form
            value|=P_NUMBER;
            break;
          case 'p':                          // ponderate form
            value|=P_PONDERATE;
            break;
          case 'h':                          // help
            value|=P_HELP;
            break;
          default:
            return P_ERROR;
        }
        b++;                                 // next char
      }
    } else {
        // option must be specify with -
        return P_ERROR;
      }

    a++;                                     // next parameter
  }      cerr << value;
  return value;
}

/**
 * Show a help screen
 */
void showHelp() {
  cout << "Examiner 1.0\n";
  cout << "Syntax: -<option>... [-<option>...]\n";
  cout << "Options:\n";
  cout << " -h          display this screen\n";
  cout << " -u          show unsorted list\n";
  cout << " -s          show sorted list\n";
  cout << " -t          show single tune list (unsorted and/or sorted)\n";
  cout << " -f          show songs (files) list (unsorted and/or sorted)\n";
  cout << " -a          show author list (unsorted and/or sorted)\n";
  cout << " -l          show order list as listen time\n";
  cout << " -n          show order list as number of listen\n";
  cout << " -p          show order list as ponderate (not yet implemented)\n";
  cout << endl;
}

int main(int argc, char** argv) {
  int result;                                // parameters results

  // get parameters value
  result=getParameter(argc, argv);

  if (result & P_ERROR) {                    // there's a error?
    cout << "Invalid parameter option. Try 'examiner -h' for a syntax help\n";
    return 1;
  }

  if (result & P_HELP) {                     // help option?
    showHelp();
    return 0;
  }

  if (result==P_NOTHING) {                   // nothing selected?
    result=P_AUTHOR | P_TUNE |               // assume default
           P_SONG | P_SORTED |
           P_TIME;
  }

  // analise the TSID directory
  examiner.analiseTSid();

  if (result & P_TUNE) {
    if (result & P_UNSORTED) {
      cout << "******** UNSORTED LIST OF SINGLE TUNES *********************\n";
      examiner.showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_TIME)) {
      examiner.sortList(SORT_FOR_TIME);
      cout << "******** LISTEN TIME SORTED LIST OF SINGLE TUNES ***********\n";
      examiner.showList(&cout);
    }

    if ((result & P_SORTED) && (result & P_NUMBER)) {
      examiner.sortList(SORT_FOR_NUMBER);
      cout << "******** NUMBER LISTEN SORTED LIST OF SINGLE TUNES *********\n";
      examiner.showList(&cout);
    }

    if ((result & P_SORTED) && (result & P_PONDERATE)) {
      //examiner.sortList(SORT_FOR_NUMBER);
      //cout << "******** PONDERATE SORTED LIST OF SINGLE TUNES *************\n";
      //examiner.showList(&cout);
    }
  }

  examiner.mergeSongList();
  if (result & P_SONG) {

    if (result & P_UNSORTED) {
      cout << "******** UNSORTED LIST OF SONGS (FILES) ********************\n";
      examiner.showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_TIME)) {
      examiner.sortList(SORT_FOR_TIME);
      cout << "******** LISTEN TIME SORTED LIST OF SONGS (FILES) **********\n";
      examiner.showList(&cout);
    }

    if ((result & P_SORTED) && (result & P_NUMBER)) {
      examiner.sortList(SORT_FOR_NUMBER);
      cout << "******** NUMBER LISTEN SORTED LIST OF SONGS (FILES) ********\n";
      examiner.showList(&cout);
    }

    if ((result & P_SORTED) && (result & P_PONDERATE)) {
      //examiner.sortList(SORT_FOR_NUMBER);
      //cout << "******** PONDERATE SORTED LIST OF SONGS (FILES) ************\n";
      //examiner.showList(&cout);
    }
  }

  examiner.mergeAuthorList();
  if (result & P_AUTHOR) {

    if (result & P_UNSORTED) {
      cout << "******** UNSORTED LIST OF AUTHORS **************************\n";
      examiner.showList(&cout);
      cout << endl;
    }

    if ((result & P_SORTED) && (result & P_TIME)) {
      examiner.sortList(SORT_FOR_TIME);
      cout << "******** LISTEN TIME SORTED LIST OF AUTHORS ****************\n";
      examiner.showList(&cout);
    }

    if ((result & P_SORTED) && (result & P_NUMBER)) {
      examiner.sortList(SORT_FOR_NUMBER);
      cout << "******** NUMBER LISTEN SORTED LIST OF AUTHORS **************\n";
      examiner.showList(&cout);
    }

    if ((result & P_SORTED) && (result & P_PONDERATE)) {
      //examiner.sortList(SORT_FOR_NUMBER);
      //cout << "******** PONDERATE SORTED LIST OF AUTHORS ******************\n";
      //examiner.showList(&cout);
    }
  }
  return 0;
}

// _Examiner.CPP
